module TYCiCore
	TEMPLATE_CONFIG = '.tuya_project_template_odm'
	URL = 'https://github.com/TuyaInc/TYProjectTemplate.git'
	class TemplateConfigurator

		attr_accessor :xcodeproj_path, :platform, :remove_demo_project
		attr_accessor :pod_name, :type, :prefix, :pod_spec, :podfile_path
		attr_accessor :date, :year, :owner_email, :owner_name
		attr_accessor :group
		attr_accessor :config

		def initialize(name, config, prefix, group)

			@pod_name = name
			@pod_spec = "#{name}.podspec"
			@podfile_path = "Example/Podfile"
			@prefix = prefix

			@group = group

			@xcodeproj_path = "#{@pod_name}/Example/PROJECT.xcodeproj"
			@platform = 'ios'
			@remove_demo_project = false

			@date = date
			@year = year
			@owner_email = user_email
			@owner_name = user_name

			@config = config
		end

		def self.load_config(folder)

			path = "#{folder}/#{TYCiCore::TEMPLATE_CONFIG}"

			raise 'Can not find config file, pls ask tuya' unless File.exist? path

			content = File.read path

			json = JSON(content)

			TemplateConfig.new json
		end

		def user_email
			(ENV['GIT_COMMITTER_EMAIL'] || `git config user.email`).strip
		end

		def user_name
			(ENV['GIT_COMMITTER_NAME'] || github_user_name || `git config user.name` || `<GITHUB_USERNAME>` ).strip
		end

		def github_user_name
			github_user_name = `security find-internet-password -s github.com | grep acct | sed 's/"acct"<blob>="//g' | sed 's/"//g'`.strip
			is_valid = github_user_name.empty? or github_user_name.include? '@'
			return is_valid ? nil : github_user_name
		end

		def year
			Time.now.year.to_s
		end

		def date
			Time.now.strftime "%m/%d/%Y"
		end

	end

	class TemplateConfig

		attr_accessor :templates_json
		attr_accessor :last_version
		attr_accessor :keys, :sub_keys
		attr_accessor :type, :alias, :project

		def initialize(json)
			@project = 'PROJECT'
			@templates_json = json['templates']
			@last_version = json['last_version']

			@sub_keys = Hash.new
			@type = ""
			@alias = ""
			@desc = ""

			@type_map = Hash.new
			@templates_json.each do |item|
				key = item['type'].downcase
				@type_map[key] = item
				sub_type_map = Hash.new
				sub_types = item['sub_types']
				if sub_types && sub_types.size > 0
					sub_types.each do |sub_item|
						sub_type_map[sub_item['type']] = sub_item
					end
					item['sub_types_map'] = sub_type_map
				end
			end
			@keys = @type_map.keys
		end

		def sub_types(key)
			sub_type_map = @type_map[key]['sub_types_map']
			if sub_type_map
				sub_type_map.keys
			else
				nil
			end
		end

		def log_type
			puts "Here is the module type list".magenta.underline
			@templates_json.each do |item|
				puts "#{item['type']} : #{item['desc']}".green
			end
		end

		def log_sub_type(type)
			puts "Here is the #{type} module type list".magenta.underline
			item = @type_map[type]
			sub_types = item['sub_types']
			if sub_types && sub_types.size > 0
				sub_types.each do |sub_item|
					puts "  #{sub_item['type']} : #{sub_item['desc']}".yellow
				end
			end
		end

		def setup(type, sub_type)
			item = @type_map[type]
			if sub_type
				item = item['sub_types_map'][sub_type]
			end
			setup_item item
		end

		def setup_item(item)
			@type = item['type']
			@alias = item['alias']
			@desc = item['desc']
		end
	end
end